#!/usr/bin/perl

# Read in lines with tokens of syntactic expansions, with two or three tab-separated fields: 1) number indicating
# line number of first word of expansion in corpus file, 2) (optional) letter indicating validity of token ("g"
# means valid), 3) 20-word context around expansion: the expansion starts with the 10th word of the context and is
# surrounded by square brackets. The first command-line argument is the length of the expansion in words; the
# second argument is the desired degree of verbosity in the output (0, 1, or 2). The script measures the repetition
# tendency of the expansion; see the paper "The Inverse-Frequency Effect: An Exploratory Study" for details.

# *************** READ IN DATA ***********************

$corpus_size = 11519975;  # Corpus size must be specified! This is the size of the Coca corpus sample used in the study.

$cons_length = $ARGV[0];

$verbosity = $ARGV[1];

if(@ARGV == 3) {
    open($infile, $ARGV[2]);
}
else {
    $infile = *STDIN;
}

$i = -1;     # Number the first item as -1 (no distance is calculated)

if($verbosity == 0) {
    printf("%s ", $ARGV[2]);
}

while(<$infile>) {

    chomp($_);

    @in = split(/\t/, $_);

    $line_num = $in[0];
	
    if(@in == 3) {
	# It's a hand-annotated file
	$text_string = $in[2];
	if($in[1] ne "g") {
	    next;
	}
    }

    else {
	# It's an unannotated file
	$text_string = $in[1];
    }
    
    $num_tokens++;
    
    @wa = split(/ /, $text_string);

    if($wa[9] ne "[") {
	# There are a few lines in the Coca input that were incorrectly formatted, causing badly-formatted lines in the expansion files; we skip these lines
	# printf("Error: misplaced '[' in: %s\n", $_);
	next;
    }
    if($wa[9 + $cons_length + 1] ne "]") {
	# printf("Error: misplaced ']' in: %s\n", $_);
	next;
    }
    
    # First bracketed word is $wa[10]
    $new1 = $wa[10];
    $new2 = $wa[11];
    $new3 = $wa[12];

    # If the bracketed constituent is identical to the previous one, skip it completely (ignore in the following analysis)
    if($i > -1) {
	if( ($cons_length == 2 && $new1 eq $prev1 && $new2 eq $prev2) || ($cons_length == 3 && $new1 eq $prev1 && $new2 eq $prev2 && $new3 eq $prev3) ) {
	    next;
	}
	$t[$i] = $line_num;
	$d[$i] = $line_num - $prev_line_num;     # For each token, $d[i$] is the distance to the previous token (that doesn't have the same words)
	$line[$i] = $text_string;
	# printf("%d\n", $d[$i]);

	if($d[$i] == $cons_length + 1 && $wa[8] =~ /^(And|and|AND|Or|or|OR)$/) {
	    if($verbosity > 1) {
		printf("Matching coord: %s\n", $line[$i]);
	    }
	    $matching_coord_count++;
	}
    }

    # printf("%d\n", $line_num);
    $prev_line_num = $line_num;
    $prev1 = $new1;
    $prev2 = $new2;
    $prev3 = $new3;
    $i++;
}

$num_intervals = $i;    # The number of word intervals!

if($verbosity > 0) {
    printf("%d tokens found, %d valid intervals\n", $num_tokens, $num_intervals);
}
else {
    printf("%d ", $num_intervals);
}

# ************ BUCKET WORD INTERVALS BY DISTANCE, COMPARE TO EXPECTED **************

# Bucket word distances in buckets by powers of 10

for($c = 0; $c<10; $c++) {
    for($i = 0; $i<$num_intervals; $i++) {
	if($d[$i] > 10 ** $c && $d[$i] <= 10 ** ($c+1)) {
	    if($verbosity > 1) {
		if($c == 0) {
		    printf("Match found, distance = %d: %s\n", $d[$i], $line[$i]);
		}
	    }
	    $count[$c]++;
	}
    }
}

if($count[1] > 0) {
    $vc_to_sc_ratio = $count[0] / $count[1];
}
else {
    if($verbosity > 0) {
	printf("WARNING: No 'somewhat close' repetitions found (11<=d<=100)! Instead we use the range 101<=d<=1000.\n");
    }
    $vc_to_sc_ratio = $count[0] / $count[2];
}

if($verbosity > 0) {
    printf("Observed distribution of intervals:\n");
    printf("%d-10: %d\n", $cons_length, $count[0]);
    for($c = 1; $c<10; $c++) {
	printf("%d-%d: %d\n", (10 ** $c) + 1, (10 ** ($c+1)), $count[$c]);
    }
    printf("Very-close tokens as proportion of total = %.3f\n", $count[0] / $num_intervals);
    printf("Very-close to somewhat-close ratio = %.3f\n", $vc_to_sc_ratio);
}
else {
    printf("%d %d %d %.3f %.3f ", $count[0], $count[1], $count[2], $count[0] / $num_intervals, $vc_to_sc_ratio);
}

# Compare to the expected distribution.
$p = $num_intervals / ($corpus_size - ($num_intervals * $cons_length));      # The P of the construction happening at a point if randomly distributed

if(verbosity > 0) {
    printf("P of construction = %.6f\n", $p);
    printf("Expected distribution:\n");
}
$cp[0] = 1.0 - ((1.0 - $p) ** (10 - ($cons_length-1)));
$exp_count[0] = $cp[0] * $num_intervals;
if($verbosity > 0) {
    printf("%d-10: %.3f\n", $cons_length, $exp_count[0]); 
}
for($c = 1; $c<10; $c++) {
    $range_top = (10 ** ($c+1)) - ($cons_length-1);	                             # The largest interval size for this bucket.
    $cp[$c] = (1.0 - ((1.0 - $p) ** $range_top));                                    # The cumulative P for everything up to $range_top
    $exp_count[$c] = ($cp[$c] - $cp[$c-1]) * $num_intervals;
    if($verbosity > 0) {
	printf("%d-%d: %.3f\n", (10 ** $c) + 1, (10 ** ($c+1)), $exp_count[$c]); # Subtract $cp[$c-1] from $cp[$c] to get cum prob for desired range
    }
}
$exp_vc_to_sc_ratio = $exp_count[0] / $exp_count[1];
	
if($verbosity > 0) {
    printf("Expected vc-to-sc ratio = %.3f\n", $exp_vc_to_sc_ratio);
    printf("Ratio of observed to expected ratios = %.3f\n", $vc_to_sc_ratio / $exp_vc_to_sc_ratio);
}
else {
    printf("%.3f ", $vc_to_sc_ratio / $exp_vc_to_sc_ratio);
}

if($count[0] > 0) {
    $coord_ratio = $matching_coord_count / $count[0];
}
else {
    $coord_ratio = -1;
}
if($verbosity > 0) {
    printf("Coordinate cases as proportion of very close cases: %.3f\n", $coord_ratio);
}
else {
    printf("%.3f\n", $coord_ratio);
}
